@@ -88,6 +88,7 @@ show_dialog_help (DialogData *data)
 						  err->message,
 						  GTK_STOCK_OK, GTK_RESPONSE_OK,
 						  NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 		
 		g_signal_connect (G_OBJECT (dialog), "response",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -117,8 +118,9 @@ file_sel_response_cb (GtkWidget   *w,
 	gboolean    junk_paths;
 	GList      *file_list;
 	char       *password;
+	const char *base_dir = NULL;
 
-	if (response == GTK_RESPONSE_CANCEL) {
+	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (data->dialog);
 		return TRUE;
 	}
@@ -153,6 +155,7 @@ file_sel_response_cb (GtkWidget   *w,
 						     _("Create _Folder"), GTK_RESPONSE_YES,
 						     NULL);
 			
+			gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_YES);
 			r = gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (GTK_WIDGET (d));
 
@@ -174,6 +177,7 @@ file_sel_response_cb (GtkWidget   *w,
 						     message,
 						     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
 						     NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
 			g_free (message);
 
 			gtk_dialog_run (GTK_DIALOG (d));
@@ -193,6 +197,7 @@ file_sel_response_cb (GtkWidget   *w,
 					     NULL,
 					     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
 					     NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
 
@@ -217,6 +222,7 @@ file_sel_response_cb (GtkWidget   *w,
 					     message,
 					     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
 					     NULL);
+		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
 		g_free (message);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
@@ -289,9 +295,13 @@ file_sel_response_cb (GtkWidget   *w,
 		window->password = g_strdup (password);
 	}
 
+	if (selected_files)
+		base_dir = window_get_current_location (window);
+
 	window_archive_extract (window,
 				file_list,
 				extract_to_dir,
+				base_dir,
 				skip_newer,
 				overwrite,
 				junk_paths,
@@ -453,6 +463,7 @@ create_extra_widget (DialogData *data)
 	data->e_password_entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (data->e_password_hbox), data->e_password_entry, TRUE, TRUE, 0);
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_password_entry), TRUE);
+	gtk_entry_set_visibility (GTK_ENTRY (data->e_password_entry), FALSE);
 	
 	data->e_view_folder_checkbutton = gtk_check_button_new_with_mnemonic (_("_Open destination folder after extraction"));
 	gtk_box_pack_start (GTK_BOX (vbox1), data->e_view_folder_checkbutton, FALSE, FALSE, 0);
@@ -483,15 +494,15 @@ dlg_extract (GtkWidget *widget,
 	data->dialog = gtk_file_chooser_dialog_new (
 				    _("Extract"),
 				    GTK_WINDOW (data->window->app),
-				    GTK_FILE_CHOOSER_ACTION_SAVE,
+				    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 				    GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 				    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				    FR_STOCK_EXTRACT, GTK_RESPONSE_OK,
 				    NULL);
 	file_sel = GTK_FILE_CHOOSER (data->dialog);
-	gtk_file_chooser_set_folder_mode (file_sel, TRUE);
 	gtk_file_chooser_set_select_multiple (file_sel, FALSE);
 	gtk_file_chooser_set_local_only (file_sel, TRUE);
+	gtk_dialog_set_default_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK);
 
 	gtk_file_chooser_set_extra_widget (file_sel, create_extra_widget (data));
 
