@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <time.h>
 #include <unistd.h>
 #include <sys/param.h>
@@ -478,21 +479,39 @@ count_chars_to_escape (const char *str,
 }
 
 
-/* escape with backslash the string @str. */
+gboolean
+strchrs (const char *str,
+	 const char *chars)
+{
+	const char *c;
+	for (c = chars; *c != '\0'; c++)
+		if (strchr (str, *c) != NULL)
+			return TRUE;
+	return FALSE;
+}
+
+
 char*
-escape_str (const char *str, 
-	    const char *meta_chars)
+escape_str_common (const char *str, 
+		   const char *meta_chars,
+		   const char  prefix,
+		   const char  postfix)
 {
 	int         meta_chars_n = strlen (meta_chars);
 	char       *escaped;
-	int         i, new_l;
+	int         i, new_l, extra_chars = 0;
 	const char *s;
 	char       *t;
 
 	if (str == NULL) 
 		return NULL;
 
-	new_l = strlen (str) + count_chars_to_escape (str, meta_chars);
+	if (prefix)
+		extra_chars++;
+	if (postfix)
+		extra_chars++;
+
+	new_l = strlen (str) + (count_chars_to_escape (str, meta_chars) * extra_chars);
 	escaped = g_malloc (new_l + 1);
 
 	s = str;
@@ -501,9 +520,11 @@ escape_str (const char *str,
 		gboolean is_bad = FALSE;
 		for (i = 0; (i < meta_chars_n) && !is_bad; i++)
 			is_bad = (*s == meta_chars[i]);
-		if (is_bad)
-			*t++ = '\\';
+		if (is_bad && prefix)
+			*t++ = prefix;
 		*t++ = *s++;
+		if (is_bad && postfix)
+			*t++ = postfix;
 	}
 	*t = 0;
 
@@ -511,11 +532,20 @@ escape_str (const char *str,
 }
 
 
+/* escape with backslash the string @str. */
+char*
+escape_str (const char *str, 
+	    const char *meta_chars)
+{
+	return escape_str_common (str, meta_chars, '\\', 0);
+}
+
+
 /* escape with backslash the file name. */
 char*
 shell_escape (const char *filename)
 {
-	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;=-");
+	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;");
 }
 
 
@@ -620,7 +650,7 @@ match_patterns (char       **patterns,
 		result = g_utf8_fnmatch (patterns[i], string, flags);
 		i++;
 	}
-	
+
 	return (result == 0);
 }
 
@@ -919,7 +949,7 @@ eat_spaces (const char *line)
 
 char **
 split_line (const char *line, 
-	    int   n_fields)
+	    int         n_fields)
 {
 	char       **fields;
 	const char  *scan, *field_end;
@@ -989,3 +1019,149 @@ get_temp_work_dir_name (void)
 
 	return result;
 }
+
+
+/* file list utils */
+
+
+gboolean
+file_list__match_pattern (const char *line, 
+			  const char *pattern)
+{
+	const char *l = line, *p = pattern;
+
+	for (; (*p != 0) && (*l != 0); p++, l++) {
+		if (*p != '%') {
+			if (*p != *l)
+				return FALSE;
+		} else {
+			p++;
+			switch (*p) {
+			case 'a':
+				break;
+			case 'n':
+				if (!isdigit (*l)) 
+					return FALSE;
+				break;
+			case 'c':
+				if (!isalpha (*l)) 
+					return FALSE;
+				break;
+			default:
+				return FALSE;
+			}
+		}
+	}
+	
+	return (*p == 0);
+}
+
+
+int
+file_list__get_index_from_pattern (const char *line, 
+				   const char *pattern)
+{
+	int         line_l, pattern_l;
+	const char *l;
+
+	line_l = strlen (line);
+	pattern_l = strlen (pattern);
+
+	if ((pattern_l == 0) || (line_l == 0))
+		return -1;
+
+	for (l = line; *l != 0; l++) 
+		if (file_list__match_pattern (l, pattern))
+			return (l - line);
+	
+	return -1;
+}
+
+
+char*
+file_list__get_next_field (const char *line,
+			   int         start_from,
+			   int         field_n)
+{
+	const char *f_start, *f_end;
+	
+	line = line + start_from;
+	
+	f_start = line;
+	while ((*f_start == ' ') && (*f_start != *line))
+		f_start++;
+	f_end = f_start;
+
+	while ((field_n > 0) && (*f_end != 0)) {
+		if (*f_end == ' ') {
+			field_n--;
+			if (field_n != 0) {
+				while ((*f_end == ' ') && (*f_end != *line))
+					f_end++;
+				f_start = f_end;
+			}
+		} else
+			f_end++;
+	}
+	
+	return g_strndup (f_start, f_end - f_start);
+}
+
+
+char*
+file_list__get_prev_field (const char *line,
+			   int         start_from,
+			   int         field_n)
+{
+	const char *f_start, *f_end;
+
+	f_start = line + start_from - 1;
+	while ((*f_start == ' ') && (*f_start != *line))
+		f_start--;
+	f_end = f_start;
+
+	while ((field_n > 0) && (*f_start != *line)) {
+		if (*f_start == ' ') {
+			field_n--;
+			if (field_n != 0) {
+				while ((*f_start == ' ') && (*f_start != *line))
+					f_start--;
+				f_end = f_start;
+			}
+		} else
+			f_start--;
+	}
+
+	return g_strndup (f_start + 1, f_end - f_start);
+}
+
+
+char *
+str_substitute (const char *str,
+		const char *from_str,
+		const char *to_str)
+{
+	char    **tokens;
+	int       i;
+	GString  *gstr;
+
+	if (str == NULL)
+		return NULL;
+
+	if (from_str == NULL)
+		return g_strdup (str);
+
+	if (strcmp (str, from_str) == 0)
+		return g_strdup (to_str);
+
+	tokens = g_strsplit (str, from_str, -1);
+
+	gstr = g_string_new (NULL);
+	for (i = 0; tokens[i] != NULL; i++) {
+		gstr = g_string_append (gstr, tokens[i]);
+		if ((to_str != NULL) && (tokens[i+1] != NULL))
+			gstr = g_string_append (gstr, to_str);
+	}
+
+	return g_string_free (gstr, FALSE);
+}
