@@ -290,7 +290,7 @@ fr_command_rar_extract (FRCommand  *comm,
 		fr_process_add_arg (comm->process, scan->data);
 
 	if (dest_dir != NULL) {
-		gchar *e_dest_dir = shell_escape (dest_dir);
+		char *e_dest_dir = fr_command_escape (comm, dest_dir);
 		fr_process_add_arg (comm->process, e_dest_dir);
 		g_free (e_dest_dir);
 	}
