@@ -74,7 +74,6 @@ typedef struct {
         GtkWidget *      app;
 	GtkWidget *      list_view;
 	GtkListStore *   list_store;
-	GtkListStore *   empty_store;
 	GtkWidget *      toolbar;
 	GtkWidget *      statusbar;
 	GtkWidget *      location_bar;
@@ -86,6 +85,9 @@ typedef struct {
 	GtkWidget *      fwd_button;
 
 	GtkTooltips     *tooltips;
+	guint            help_message_cid;
+	guint            list_info_cid;
+	guint            progress_cid;
 
 	GtkWidget *      up_arrows[5];
 	GtkWidget *      down_arrows[5];
@@ -126,6 +128,9 @@ typedef struct {
 	gint             activity_ref;              /* when > 0 some activity
                                                      * is present. */
 
+	guint            update_timeout_handle;     /* update file list 
+						     * timeout handle. */
+
 	VisitDirHandle  *vd_handle;
 
 	FRConvertData    convert_data;
@@ -136,58 +141,12 @@ typedef struct {
 	guint            clipboard_op : 1;
 	char            *clipboard_current_dir;
 
-	/* Menu items. */
+	GtkActionGroup  *actions;
 
-	GtkWidget *      mitem_new_archive;
-	GtkWidget *      mitem_open_archive;
-	GtkWidget *      mitem_save_as_archive;
-	GtkWidget *      mitem_close;
-	GtkWidget *      mitem_recents_menu;
-
-	GtkWidget *      mitem_archive_prop;
-	GtkWidget *      mitem_move_archive;
-	GtkWidget *      mitem_copy_archive;
-	GtkWidget *      mitem_rename_archive;
-	GtkWidget *      mitem_delete_archive;
-
-	GtkWidget *      mitem_cut;
-	GtkWidget *      mitem_copy;
-	GtkWidget *      mitem_paste;
-	GtkWidget *      mitem_select_all;
-	GtkWidget *      mitem_unselect_all;
-	GtkWidget *      mitem_rename;
-
-	GtkWidget *      mitem_view_toolbar;
-	GtkWidget *      mitem_view_statusbar;
-	GtkWidget *      mitem_view_flat;
-	GtkWidget *      mitem_view_as_dir;
-	GtkWidget *      mitem_sort[5];
-	GtkWidget *      mitem_sort_reversed;
-	GtkWidget *      mitem_password;
-	GtkWidget *      mitem_last_output;
-
-	GtkWidget *      mitem_add;
-	GtkWidget *      mitem_delete;
-	GtkWidget *      mitem_extract;
-	GtkWidget *      mitem_test;
-	GtkWidget *      mitem_view;
-	GtkWidget *      mitem_open;
-	GtkWidget *      mitem_stop;
-	GtkWidget *      mitem_reload;
-
-	GtkWidget *      toolbar_new;
-	GtkWidget *      toolbar_open;
-	GtkWidget *      toolbar_add;
-	GtkWidget *      toolbar_delete;
-	GtkWidget *      toolbar_extract;
-	GtkWidget *      toolbar_view;
-	GtkWidget *      toolbar_stop;
-
-	GtkWidget *      file_popup_menu;
-	GtkWidget *      popupmenu_file[14];
-
-	EggRecentViewGtk *recent_view;
+	EggRecentViewUIManager *recent_view;
 	EggRecentModel   *recent_model;
+	GtkWidget        *file_popup_menu;
+	GtkWidget        *mitem_recents_menu;
 
 	/* drag data */
 
@@ -279,6 +238,7 @@ void       window_archive_add_with_wildcard (FRWindow      *window,
 					     const char    *include_files,
 					     const char    *exclude_files,
 					     const char    *base_dir,
+					     const char    *dest_dir,
 					     gboolean       update,
 					     gboolean       recursive,
 					     gboolean       follow_links,
@@ -288,6 +248,7 @@ void       window_archive_add_with_wildcard (FRWindow      *window,
 void       window_archive_add_directory     (FRWindow      *window,
 					     const char    *directory,
 					     const char    *base_dir,
+					     const char    *dest_dir,
 					     gboolean       update,
 					     const char    *password,
 					     FRCompression  compression);
@@ -303,6 +264,7 @@ void       window_archive_remove            (FRWindow      *window,
 void       window_archive_extract           (FRWindow      *window,
 					     GList         *file_list,
 					     const char    *extract_to_dir,
+					     const char    *base_dir,
 					     gboolean       skip_older,
 					     gboolean       overwrite,
 					     gboolean       junk_paths,
